<?php
/*
Plugin Name: Splinder Importer
Plugin URI: http://www.aioros.net/splinder-importer-2010/
Description: Import posts, comments and categories from a Splinder blog.
Author: Stefano Morciano, Brian Colinger
Author URI: http://www.aioros.net/
Version: 0.4
License: GPL v3 - http://www.gnu.org/licenses/gpl-3.0.html
*/

if ( !defined( 'WP_LOAD_IMPORTERS' ) )
	return;

// Load Importer API
require_once ABSPATH . 'wp-admin/includes/import.php';

if ( !class_exists( 'WP_Importer' ) ) {
	$class_wp_importer = ABSPATH . 'wp-admin/includes/class-wp-importer.php';
	if ( file_exists( $class_wp_importer ) )
		require_once $class_wp_importer;
}

/**
 * Splinder Importer
 *
 * @package WordPress
 * @subpackage Importer
 */
class Splinder_Import extends WP_Importer {
	var $importer_code = 'splinder-importer';
	var $importer_name = 'Splinder';
	var $importer_desc = '';
	var $today;
	var $totalposts = 0;
	var $totalcomments = 0;
	var $verboseposts = "";
	var $verbosecoms = "";

	/**
	 * Class constructor
	 *
	 * @return void
	 */
	function __construct() {
		$this->today = getdate();
		$this->importer_desc = __( 'Import a Splinder blog', 'splinder-importer' );
	}

	/**
	 * PHP 4 style constuctor
	 *
	 * @return void
	 */
	function Splinder_Import() {
		$this->__construct();
	}

	/**
	 * Print header
	 * 
	 * @return void
	 */
	function header() {
		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>' . __( 'Import Splinder', 'splinder-importer' ) . '</h2>';
	}

	/**
	 * Print footer
	 * 
	 * @return void
	 */
	function footer() {
		echo '</div>';
	}

	/**
	 * Print greeting
	 * 
	 * @return void
	 */
	function greet() {
		echo '<div class="narrow">';
		echo '<p>' . __( "Prepare your template, then set the URL, the start date and the end date.", 'splinder-importer' ) . '</p>';
		$this->import_splinder_form( "admin.php?import=" . $this->importer_code . "&amp;step=1" );
		echo '</div>';
	}
	
	/**
	 * Import calls each step of the import process
	 * 
	 * @param string $blogurl
	 * @param string $start_month
	 * @param string $start_year
	 * @param string $end_month
	 * @param string $end_year
	 * @return void
	 */
	function import( $blogurl, $start_month, $start_year, $end_month, $end_year, $tagsorcats, $uncategorized, $othercat, $verboseposts, $verbosecoms ) {
		$this->verboseposts = $verboseposts;
		$this->verbosecoms = $verbosecoms;
		$month = $start_month;
		$year = $start_year;
		$start_time = time();
		$max_time = ini_get( 'max_execution_time' );
		while ( ( $year < $end_year ) || ( ( $year == $end_year ) && ( $month <= $end_month ) ) ) {
			//echo time() . " - $start_time<br />max = $max_time<br />";
			if ( ($max_time > 0) && (time() - $start_time > 0.7 * $max_time ) ) {
				echo "<p><h3><a href=\"admin.php?import=" . $this->importer_code
					. "&amp;step=2&amp;bu={$blogurl}&amp;sm={$month}&amp;sy={$year}&amp;em={$end_month}&amp;ey={$end_year}"
					. "&amp;tc={$tagsorcats}&amp;un={$uncategorized}&amp;ot={$othercat}"
					. "&amp;vp={$verboseposts}&amp;vc={$verbosecoms}"
					. "\">"
					. __( 'Continue importing...', 'splinder-importer' ) . "</a></h3></p>";
				return;
			} else {
			
				echo "<br />" . sprintf( "%04d-%02d", $year, $month ) . "<br />";
				$url = $blogurl . "/archive/" . sprintf( "%04d-%02d", $year, $month );
				$data = $this->get_page( $url );
				if ( is_wp_error( $data ) ) {
					echo __( 'Error:', 'splinder-importer' ) . "\n" . $data->get_error_message() . "\n";
					return false;
				}
				$html = $data['body'];
				$inputposts = explode( "\$\$\$POSTSEPARATOR\$\$\$", $html );
				array_splice( $inputposts, 0, 1 );
				array_splice( $inputposts, -1, 1 );
				$inputposts = array_reverse( $inputposts );
				$this->totalposts = $this->totalposts + count( $inputposts );
				echo __( 'Posts:', 'splinder-importer' ) . ' ' . count( $inputposts ) . "<br /><br />";

				foreach ( $inputposts as $post ) {
					list ( , $number, ) = explode( "\$\$\$ITEMNUMBER\$\$\$", $post );
					$postpage_data = $this->get_page( $blogurl . "/post/" . $number . "#comment" );
					if ( is_wp_error( $postpage_data ) ) {
						echo __( 'Error:', 'splinder-importer' ) . "\n" . $postpage_data->get_error_message() . "\n";
						return false;
					}
					$postpage = $postpage_data['body'];
					list ( , $datetime, ) = explode( "\$\$\$ITEMDATETIME\$\$\$", $postpage );
					preg_match( "#([0-9]{2})/([0-9]{2})/([0-9]{2,4}) ([0-9]{2}:[0-9]{2})#", $datetime, $datetime );
					$datetime = $datetime[3] . "-" . $datetime[2] . "-" . $datetime[1] . " " . $datetime[4] . ":00";
					list ( , $categories, ) = explode( "\$\$\$ITEMCATS\$\$\$", $postpage );
					list ( , $title, ) = explode( "\$\$\$ITEMTITLE\$\$\$", $postpage );
					preg_match( "#<a href=[^<]+>([^<]+)</a>#", $title, $title );
					$title = $title[1];
					list ( , $body, ) = explode( "\$\$\$ITEMBODY\$\$\$", $postpage );
					$body = substr( $body, 0, strpos( $body, "<!--" ) );
					list ( , $author, ) = explode( "\$\$\$ITEMAUTHOR\$\$\$", $postpage );
					list ( , $comments, ) = explode( "\$\$\$COMMENTS\$\$\$", $postpage );
					$comDates = $this->splinder_com_dates( $comments );
					$comBodies = $this->splinder_com_texts( $comments );
					$comAuthors = $this->splinder_com_authors( $comments );

					// insert author
					$author_id = $this->add_user( $author );

					// insert categories and tags
					$cat_ids = array();
					$tag_ids = array();
					$tag_names = array();

					if ( $tagsorcats == "tags" ) {
						$ntags = preg_match_all( "#tag'>([^<]+)</a>#Us", $categories, $cleantags );
						if ( $ntags > 0 ) {
							for ( $i = 0; $i < count( $cleantags[0] ); $i++ ) {
								$tag_ids[$i] = $this->add_tag( $cleantags[1][$i] );
								$tag_ids[$i] = $tag_ids[$i]['term_id'];
								$tag_names[$i] = $cleantags[1][$i];
							}
						}
						$tag_names = implode( ",", $tag_names );
						if ( $uncategorized == "default" )
							$cat_ids = array( get_option( 'default_category' ) );
						else if ( $uncategorized == "other" )
							$cat_ids = array( $this->add_category( $othercat ) );
					}

					if ( $tagsorcats == "cats" ) {
						$ncats = preg_match_all( "#tag'>([^<]+)</a>#Us", $categories, $cleancats );
						if ( $ncats > 0 ) {
							for ( $i = 0; $i < count( $cleancats[0] ); $i++ ) {
								$cat_ids[$i] = $this->add_category( $cleancats[1][$i] );
							}
						} else {
							if ( $uncategorized == "default" )
								$cat_ids = array( get_option( 'default_category' ) );
							else if ( $uncategorized == "other" )
								$cat_ids = array( $this->add_category( $othercat ) );
						}
					}

					// insert post
					$post_id = $this->add_post( $title, $body, $author_id, $datetime, $cat_ids, $tag_names );

					// insert comments
					for ( $i = 0; $i < count( $comDates[0] ); $i++ ) {
						$comDate = $comDates[3][$i] . "-" . $this->month_numberize( $comDates[2][$i] ) . "-" . $comDates[1][$i] . " " . $comDates[4][$i] . ":00";
						$comment_id = $this->add_comment( $post_id, $comBodies[1][$i], $comAuthors[3][$i], $comAuthors[2][$i], $comDate );
					}
					$this->totalcomments = $this->totalcomments + count( $comDates[0] );

				}

				$month = $month % 12 + 1; // - what, that actually works? - i know, right?
				$year = $year + ( $month == 1 );
				
			}
		}

		echo '<br />';
		echo __( 'Total posts:', 'splinder-importer' ) . ' ' . $this->totalposts;
		echo '<h3>' . sprintf( __( 'All done. %sHave fun!%s', 'splinder-importer' ), "<a href=\"" . get_option( 'home' ) . "\">", "</a>") . '</h3>';
	}

	/**
	 * Code from Terenzani's plugin (http://www.terenzani.it/38/importare-i-post-di-splinder-in-wordpress/)
	 * Example: $comDates[i][j] == i-th datetime element (from regex capturing group) of the j-th comment
	 *
	 * @param string $comments
	 * @return string | bool false on fail
	 */
	function splinder_com_dates( $comments ) {
		preg_match_all( "#:right\">([0-9]{1,2}) ([\w]+) ([0-9]{4}) - ([0-9]{2}:[0-9]{2})#", $comments, $comDates );
		if ( !empty( $comDates ) )
			return $comDates;
		return false;
	}

	/**
	 * Return comment texts
	 * 
	 * @param mixed $comments
	 * @return mixed
	 */
	function splinder_com_texts( $comments ) {
		preg_match_all( "#<div class=\"comment-body\">(.+)</div>#Us", $comments, $comBodies );
		if ( !empty( $comBodies ) )
			return $comBodies;
		return false;
	}

	/**
	 * Return comment authors
	 * 
	 * @param string $comments
	 * @return array
	 */
	function splinder_com_authors( $comments ) {
		preg_match_all( "#<td class=\"icons\">[\r\n\t]+(<a href=\"(.+)\".+)?</td>[\r\n\t]+<td>[\r\n\t]+([\w\d \._-]+)[\r\n\t]+</td>#Us", $comments, $comAuthors );
		if ( empty( $comAuthors[0] ) ) {
			preg_match_all( "#<div class=\"comment-bottom\">[\r\n\t]+([\w\d \._-]+)[\r\n\t]+.+(<a href=\"(http://(?!www\.motime\.com).+)\"|<br />)#Us", $comments, $comAuthors );
			$comAuthors[2] = $comAuthors[3];
			$comAuthors[3] = $comAuthors[1];
		}
		if ( !empty( $comAuthors ) )
			return $comAuthors;
		return false;
	}

	/**
	 * Convert month name to month number
	 * 
	 * @param string $string
	 * @return int
	 */
	function month_numberize( $string ) {
		$a1 = array( "gennaio", "febbraio", "marzo", "aprile", "maggio", "giugno", "luglio", "agosto", "settembre", "ottobre", "novembre", "dicembre" );
		$a3 = array( 'january', 'february', 'mars', 'april', 'may', 'june', 'july', 'august', 'september', 'october', 'november', 'december' );
		$a2 = array( 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 );
		$string = str_replace( $a1, $a2, strtolower( $string ) );
		return $string = str_replace( $a3, $a2, strtolower( $string ) );
	}

	/**
	 * Convert month number to month name
	 * 
	 * @param int $month
	 * @return string
	 */
	function month_tostring( $month ) {
		$a1 = array( __( 'October', 'splinder-importer' ), __( 'November', 'splinder-importer' ), __( 'December', 'splinder-importer' ),
					__( 'January', 'splinder-importer' ), __( 'February', 'splinder-importer' ), __( 'March', 'splinder-importer' ),
					__( 'April', 'splinder-importer' ), __( 'May', 'splinder-importer' ), __( 'June', 'splinder-importer' ),
					__( 'July', 'splinder-importer' ), __( 'August', 'splinder-importer' ), __( 'September', 'splinder-importer' ) );
		$a2 = array( 10, 11, 12, 1, 2, 3, 4, 5, 6, 7, 8, 9 );
		return str_replace( $a2, $a1, $month );
	}

	/**
	 * Code from IoBloggo importer (http://www.andreabeggi.net/2008/10/30/importare-da-iobloggo-a-wordpress/)
	 * 
	 * @param string $username
	 * @return int
	 */
	function add_user( $username ) {
		$user_id = username_exists( $username );
		if ( !$user_id ) {
			$admin_info = get_userdata( 1 );
			if ( $username == $admin_info->nickname )
				return 1;
			$pass = substr( md5( uniqid( microtime() ) ), 0, 6 );
			$user_id = wp_create_user( $username, $pass );
			printf( __( 'Created user %s', 'splinder-importer' ) . "<br />", "<i>$username</i>" );
		}
		return $user_id;
	}

	/**
	 * Add Category
	 * 
	 * @param string $catname
	 * @return int
	 */
	function add_category( $catname ) {
		$category_id = category_exists( $catname );
		if ( !$category_id ) {
			$category_id = wp_create_category( $catname );
			//printf( __( 'Created category %s', 'splinder-importer' ) . "<br />", "<i>$catname</i>" );
		}
		return $category_id;
	}

	/**
	 * Add Tag
	 * 
	 * @param string $tagname
	 * @return int
	 */
	function add_tag( $tagname ) {
		$tag_id = tag_exists( $tagname );
		if ( !$tag_id ) {
			$tag_id = wp_create_tag( $tagname );
			//printf( __( 'Created tag %s', 'splinder-importer' ) . "<br />", "<i>$tagname</i>" );
		}
		return $tag_id;
	}

	/**
	 * Add Post
	 * 
	 * @param string $title
	 * @param string $content
	 * @param string $author
	 * @param string $date
	 * @param string $categories
	 * @param string $tags
	 * @return int
	 */
	function add_post( $title, $content, $author, $date, $categories, $tags ) {
		$post_id = $this->post_exists( $author, $date );
		if ( !$post_id ) {
			$post = array();
			$post['post_title'] = $title;
			$post['post_content'] = $content;
			$post['post_author'] = $author;
			$post['post_date'] = $date;
			$post['post_category'] = $categories;
			$post['tags_input'] = $tags;
			$post['post_status'] = "publish";
			$post_id = wp_insert_post( $post );
			if ( $this->verboseposts == "posts" )
				printf( __( 'Added post %s', 'splinder-importer') . "<br />", "<i>$title</i>, " . get_userdata($author)->nickname . ", $date" );
		}
		return $post_id;
	}

	/**
	 * Add Comment
	 * 
	 * @param int $post_id
	 * @param string $content
	 * @param string $author
	 * @param string $author_url
	 * @param string $date
	 * @return int
	 */
	function add_comment( $post_id, $content, $author, $author_url, $date ) {
		$comment_id = $this->comment_exists( $author, $date );
		if ( !$comment_id ) {
			$comment = array();
			$comment['comment_post_ID'] = $post_id;
			$comment['comment_content'] = $content;
			$comment['comment_author'] = $author;
			$comment['comment_author_url'] = $author_url;
			$comment['comment_date'] = $date;
			$comment = wp_filter_comment( $comment );
			$comment_id = wp_insert_comment( $comment );
			if ( $this->verbosecoms == "comments" )
				printf( __( 'Added comment by %1$s on %2$s', 'splinder-importer' ) . "<br />", $author, $date );
		}
		return $comment_id;
	}

	/**
	 * Check if a post exists
	 * 
	 * @param string $author
	 * @param string $date
	 * @return bool
	 */
	function post_exists( $author, $date ) {
		global $wpdb;
		$query = $wpdb->prepare( "SELECT * FROM $wpdb->posts WHERE post_author = %d AND post_date = %s", $author, $date );
		$result = $wpdb->get_results( $query );
		if ( count( $result ) > 0 ) {
			$result = $result[0];
			if ( $this->verboseposts == "posts" )
				printf( __( "Post by %1$s on %2$s already exists", 'splinder-importer' ) . "<br />", get_userdata( $author )->user_login, $date );
			return $result->ID;
		}
		return FALSE;
	}

	/**
	 * Check if a comment exists
	 * 
	 * @param string $author
	 * @param string $date
	 * @return bool
	 */
	function comment_exists( $author, $date ) {
		global $wpdb;
		$query = $wpdb->prepare( "SELECT * FROM $wpdb->comments WHERE comment_author = %d AND comment_date = %s", $author, $date );
		$result = $wpdb->get_results( $query );
		if ( count( $result ) > 0 ) {
			$result = $result[0];
			return $result->comment_ID;
		}
		return FALSE;
	}

	/**
	 * Print the splinder form
	 * 
	 * @param string $action
	 * @return void
	 */
	function import_splinder_form( $action ) {
?>
		<form enctype="multipart/form-data" id="import-form" method="post" action="<?php echo esc_attr( wp_nonce_url( $action, 'import-splinder-form' ) ); ?>">
		<p>
		<label for="template"><?php _e( 'Change your Splinder blog template with the following:', 'splinder-importer' ); ?></label>
		<textarea id="template" name="template" cols="80" rows="8" readonly="readonly">&lt;html&gt;&lt;body&gt;<?php _e( 'Work in progress...', 'splinder-importer' ); ?>&lt;BR&gt;&lt;BR&gt;&lt;BR&gt;&lt;BR&gt;&lt;BR&gt;
<?php _e( 'EXPORTING BLOG...', 'splinder-importer' ); ?>&lt;BR&gt;
&lt;Blogger&gt;$$$POSTSEPARATOR$$$
$$$ITEMNUMBER$$$&lt;$BlogItemNumber$&gt;$$$ITEMNUMBER$$$
$$$ITEMDATETIME$$$&lt;$BlogItemDateTime$&gt;$$$ITEMDATETIME$$$
$$$ITEMCATS$$$&lt;$BlogItemCategories$&gt;$$$ITEMCATS$$$
$$$ITEMTITLE$$$&lt;$BlogItemTitle$&gt;$$$ITEMTITLE$$$
$$$ITEMBODY$$$&lt;$BlogItemBody$&gt;$$$ITEMBODY$$$
$$$ITEMAUTHOR$$$&lt;$BlogItemAuthor$&gt;$$$ITEMAUTHOR$$$
$$$COMMENTS$$$&lt;$BlogItemCommentList$&gt;$$$COMMENTS$$$
&lt;/Blogger&gt;$$$POSTSEPARATOR$$$
&lt;$BlogLinkOut$&gt;
&lt;/body&gt;&lt;/html&gt;</textarea>
		</p><p>
		<?php _e( "From your Splinder control panel, disable blog pager (Blogs -&gt; Settings -&gt; Format -&gt; Enable blog pager -&gt; No) and set \"Format of time stamp\" as in \"", 'splinder-importer' ); ?><script type="text/javascript">
		<!--
		var currentTime = new Date();
		var month = currentTime.getMonth() + 1;
		var day = currentTime.getDate();
		var year = currentTime.getFullYear();
		var hours = currentTime.getHours()
		var minutes = currentTime.getMinutes()
		if (day < 10) day = "0" + day;
		if (month < 10) month = "0" + month;
		if (hours < 10) hours = "0" + hours;
		if (minutes < 10) minutes = "0" + minutes;
		document.write(day + "/" + month + "/" + year + " " + hours + ":" + minutes);
		//-->
		</script>".
		</p><p>
		<label for="import-url"><?php echo __( 'Blog URL:', 'splinder-importer' ) . ' '; ?></label>http://
		<input type="text" id="import-url" name="import-url" size="20" />.splinder.com
		</p><p>
		<label for="start-month"><?php _e( 'From:', 'splinder-importer' ); ?></label>
		<select id="start-month" name="start-month">
			<?php
			for ( $i = 1; $i <= 12; $i++ ) {
				echo "<option value=\"$i\"";
				if ( 1 == $i )
					echo ' selected="selected"';
				echo ">" . $this->month_tostring( $i ) . "</option>";
			}
			?>
		</select>
		<input type="text" id="start-year" name="start-year" value="2001" size="5" />
		</p><p>
		<label for="end-month"><?php _e( 'To:', 'splinder-importer' ); ?></label>
		<select id="end-month" name="end-month">
			<?php
			for ( $i = 1; $i <= 12; $i++ ) {
				echo "<option value=\"$i\"";
				if ( $i == $this->today['mon'] )
					echo ' selected="selected"';
				echo ">" . $this->month_tostring( $i ) . "</option>";
			}
			?>
		</select>
		<input type="text" id="end-year" name="end-year" value="<?php echo $this->today['year']; ?>" size="5" />
		</p><p>
		<label for="tagsorcats"><?php _e( 'Splinder categories:', 'splinder-importer' ); ?></label><br />
		<input type="radio" id="tagsorcats" name="tagsorcats" value="tags" checked="checked" /><?php _e( 'Import as WordPress tags', 'splinder-importer' ); ?><br />
		<input type="radio" id="tagsorcats" name="tagsorcats" value="cats" /><?php _e( 'Import as WordPress categories', 'splinder-importer' ); ?><br />
		</p><p>
		<label for="uncategorized"><?php _e( 'Posts without a category:', 'splinder-importer' ); ?></label><br />
		<input type="radio" id="uncategorized" name="uncategorized" value="default" checked="checked" /><?php _e( 'Assign to my WordPress default category', 'splinder-importer' ); ?><br />
		<input type="radio" id="uncategorized" name="uncategorized" value="other" /><?php _e( 'Assign to category...', 'splinder-importer' ); ?>
		<input type="text" id="other-category" name="other-category" size="20" />
		</p><p>
		<input type="checkbox" id="verbose-posts" name="verbose-posts" value="posts" checked="checked" /><?php _e( 'List all imported posts', 'splinder-importer' ); ?><br />
		<input type="checkbox" id="verbose-comments" name="verbose-comments" value="comments" /><?php _e( 'List all imported comments', 'splinder-importer' ); ?><br />
		</p>
		<p class="submit">
		<input type="submit" class="button" value="<?php esc_attr_e( "Start import", 'splinder-importer' ); ?>" />
		</p>
		</form>
		<?php
	}

	/**
	 * Callback for register_importer. This function will figure out what to do when the importer is initialized.
	 * 
	 * @return void
	 */
	function dispatch() {
		$step = isset( $_GET['step'] ) ? (int) $_GET['step'] : 0;

		$this->header();
		switch ( $step ) {
			case 0:
				$this->greet();
				break;
			case 1:
				check_admin_referer( 'import-splinder-form' );
				$this->import( "http://" . $_POST['import-url'] . ".splinder.com",
								$_POST['start-month'], $_POST['start-year'], $_POST['end-month'], $_POST['end-year'],
								$_POST['tagsorcats'], $_POST['uncategorized'], $_POST['other-category'],
								$_POST['verbose-posts'], $_POST['verbose-comments'] );
				break;
			case 2:
				$this->import( $_GET['bu'], $_GET['sm'], $_GET['sy'], $_GET['em'], $_GET['ey'],
								$_GET['tc'], $_GET['un'], $_GET['ot'], $_GET['vp'], $_GET['vc'] );
				break;
		}
		$this->footer();
	}
}

$splinder_import = new Splinder_Import();
register_importer( $splinder_import->importer_code, $splinder_import->importer_name, $splinder_import->importer_desc, array( &$splinder_import, 'dispatch' ) );

function splinder_importer_init() {
	load_plugin_textdomain( 'splinder-importer', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}
add_action( 'init', 'splinder_importer_init' );
